@@ -1,6 +1,7 @@
 """Declares :class:`Client`."""
 import abc
 import contextlib
+import logging
 
 import aiohttp
 from unimatrix.conf import settings
@@ -11,6 +12,7 @@ from .credentials.icredentials import ICredentials
 
 
 class Client(metaclass=abc.ABCMeta):
+    logger = logging.Logger = logging.getLogger('uvicorn')
     audience: str = abc.abstractproperty()
     scope: set = abc.abstractproperty()
 
@@ -19,6 +21,9 @@ class Client(metaclass=abc.ABCMeta):
 
     async def fromjson(self, response: aiohttp.ClientResponse) -> ImmutableDTO:
         """Await the response body and deserialize it."""
+        if response.status in (204, 304)\
+        or (response.method == 'HEAD'):
+            return None
         return ImmutableDTO.fromdict(await response.json())
 
     async def request(self, fn, *args, **kwargs):
@@ -27,16 +32,20 @@ class Client(metaclass=abc.ABCMeta):
         deserialize = kwargs.pop('deserialize', True)
         kwargs.setdefault('ssl', settings.ENABLE_SSL)
         response = await fn(*args, **kwargs)
-        if 'X-Canonical-Exception' in response.headers:
-            dto = await self.fromjson(response)
-            if retry or dto.code not in ('CREDENTIAL_EXPIRED', 'TRUST_ISSUES'):
-                exception = CanonicalException(**dto)
+        if 'X-Error-Code' in response.headers:
+            code = response.headers['X-Error-Code']
+            if retry or code != 'CREDENTIAL_EXPIRED':
+                params = (await self.fromjson(response)) or {
+                    'code': code
+                }
+                exception = CanonicalException(**params)
                 exception.http_status_code = response.status
                 raise exception
 
             # If the code is CREDENTIAL_EXPIRED or TRUST_ISSUES, then refresh
             # the access token and retry the request.
             kwargs['retry'] = True
+            kwargs['deserialize'] = deserialize
             await self.refresh_credentials()
             return await self.request(fn, *args, **kwargs)
         if not allow_failure:
@@ -49,10 +58,6 @@ class Client(metaclass=abc.ABCMeta):
         return await self.request(self.session.get, *args, **kwargs)
 
     async def head(self, *args, **kwargs):
-        kwargs.update({
-            'deserialize': False,
-            'allow_failure': True
-        })
         return await self.request(self.session.head, *args, **kwargs)
 
     async def patch(self, *args, **kwargs):
